@@ -9,21 +9,16 @@
 """This module contains the command line interface"""
 
 import argparse
-import logging
+import asyncio
+import logging.config
+from QKDSimkit import LOGGING
 
-from aiocache import Cache
+logging.config.dictConfig(LOGGING)
 
 from QKDSimkit.Channel import start_channel
 from QKDSimkit.Client import start_client
 from QKDSimkit.p2p_servers import start_p2p
-from QKDSimkit.Server import start_server, start_server_and_channel, get_key_cli
-
-logging.Logger("QKDSimkit_logger")
-logger = logging.getLogger("QKDSimkit_logger")
-logger.setLevel(logging.DEBUG)
-ch = logging.StreamHandler()
-ch.setLevel(logging.DEBUG)
-logger.addHandler(ch)
+from QKDSimkit.Server import start_server, start_server_and_channel, get_key_cli, add_user
 
 
 def cli():
@@ -41,8 +36,10 @@ def cli():
     channels = server.add_subparsers(title='Action', dest='action',
                                      help='Choose a possible action')
     parser_k = channels.add_parser('retrieve', help="Retrieve keys")
-    parser_k.add_argument('-i', '--identifier', type=str, default=None,
+    parser_k.add_argument('-t', '--token', type=str, default=None,
                           help='Specify an identifier to retrieve a specific set of keys')
+    parser_u = channels.add_parser('add_user', help='Register a new user with a token')
+    parser_u.add_argument('token', help='Specify a token to identify a client')
     parser_l = channels.add_parser('local', help='Run the channel on this machine')
     parser_l.add_argument('-ca', '--channel_address', default=':5000', type=str,
                           help='Bind socket to this address (default: %(default)s)')
@@ -61,6 +58,8 @@ def cli():
     client.add_argument('channel_address', type=str, help='Address of channel [host:port]')
     client.add_argument('-n', '--number', default=1, type=int, help="Number of keys (default: %(default)s)")
     client.add_argument('-s', '--size', default=256, type=int, help="Size of keys (default: %(default)s)")
+    client.add_argument('-t', '--token', default='token', help='Specify token')
+    client.add_argument('-k', '--show_keys', default=False, type=bool, help='Show keys in output')
 
     #   CHANNEL PARSER
     #   ==============
@@ -88,12 +87,14 @@ def cli():
     if args.program == 'server':
         if args.action == 'retrieve':
             get_key_cli(args.identifier)
+        if args.action == 'add_user':
+            asyncio.run(add_user(args.token))
         if args.action == 'local':
             start_server_and_channel(args.channel_address, args.noise, args.eve, args.address)
         if args.action == 'external':
             start_server(args.channel_address, args.address)
     elif args.program == 'client':
-        start_client(args.alice_address, args.channel_address, args.number, args.size)
+        start_client(args.alice_address, args.channel_address, args.number, args.size, args.token, args.show_keys)
     elif args.program == 'channel':
         start_channel(args.address, args.noise, args.eve)
     elif args.program == 'p2p':
